/*
 * Copyright (C) 1996, 1997, 1998,
 *
 * Department of Computing Science,
 * The University,
 * Newcastle upon Tyne,
 * UK.
 *
 * $Id: SimulationProcess.java,v 1.3 1998/12/07 08:28:11 nmcl Exp $
 */

package arjuna.JavaSim.Simulation;

import java.lang.Thread;
import java.util.NoSuchElementException;
import java.lang.InterruptedException;

public class SimulationProcess extends Thread {
    
    /** 
	Mtodo destructor de la clase SimulationProcess.
	Pasa el objeto proceso al estado terminado, el objeto 
	proceso que lo ejecuta no podr	volver al estado activo, ni al
	planificado, ni al pasivo.
    */
    
    public void finalize () {
	if (!Terminated) {
	    Terminated = Passivated = true;
	    wakeuptime = SimulationProcess.Never;
	    
	    if (!idle())
		Scheduler.unschedule(this); // remove from scheduler queue
	    
	    if (this == SimulationProcess.Current) {
		try {
		    Scheduler.schedule();
		}
		catch (SimulationException e){}
	    }
	    SimulationProcess.allProcesses.Remove(this);
	}
    }
   

    /**
       Devuelve el tiempo absoluto actual de simulacin que marca el reloj
       de tiempo virtual de la simulacin.

       @return double
    */

    public final double Time () {
	return SimulationProcess.CurrentTime();
    }


    /**
       Retorna el siguiente objeto proceso al que ejecuta este mtodo.
       Este mtodo se ejecuta sobre un objeto proceso en el estado activo 
       o en el estado planificado. 

       @return SimulationProcess Devuelve un objeto proceso de una subclase 
       de 'SimulationProcess'.
       @exception SimulationException El objeto proceso sobre el que se 
       llama este mtodo no est en el estado activo, ni planificado.
       @exception NoSuchElementException La cola de procesos planificados 
       est vaca o el que ha ejecutado el mtodo es el ltimo.
    */
    
    public synchronized SimulationProcess next_ev () throws SimulationException, NoSuchElementException {
	if (!idle())
	    // devuelve el siguiente del que ejecuta este mtodo.
	    return Scheduler.ReadyQueue.getNext(this);
	else
	    throw(new SimulationException("SimulationProcess not on run queue."));
    }


    /**
       Devuelve el tiempo absoluto de simulacin en el que se activar el 
       proceso. Si no est planificado, devuelve el valor de la 
       constante 'Never'.

       @return double
    */
    
    public final double evtime () {
	return wakeuptime;
    }
    
    /**
       Planifica el proceso antes que el objeto proceso que recibe como 
       parmetro, en el mismo tiempo absoluto de simulacin que tiene el 
       parmetro.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado activo o planificado.
       El objeto proceso que recibe como parmetro solo puede estar en el 
       estado planificado.
       
       @param p Objeto de una subclase de 'SimulationProcess'.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro es un objeto proceso 
       que no est en el estado planificado.
       @exception RestartException Si se est reiniciando.
    */

    public void ActivateBefore (SimulationProcess p) throws SimulationException, RestartException {
	if (Terminated || !idle()) return;
	
	Passivated = false;
	
	if (Scheduler.ReadyQueue.InsertBefore(this, p))
	    wakeuptime = p.wakeuptime;
	else
	    // el ejemplo 5 produce la excepcin
	    throw new SimulationException("'before' process is not scheduled.");
    }
    
     /**
       Planifica el proceso para que vaya a continuacin del objeto proceso que recibe como 
       parmetro, para en el mismo tiempo absoluto de simulacin que tiene ste ltimo.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.
       El objeto proceso que recibe como parmetro debe estar en el 
       estado planificado.
       
       @param p Objeto de una subclase de 'SimulationProcess'.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro es un objeto proceso 
       que no est en el estado planificado.
       @exception RestartException Si se est reiniciando.
     */
    
    public void ActivateAfter (SimulationProcess p) throws SimulationException, RestartException {
	if (Terminated || !idle()) return;
	
	Passivated = false;
	
	if (Scheduler.ReadyQueue.InsertAfter(this, p))
	    wakeuptime = p.wakeuptime;
	else
	    throw new SimulationException("'after' process is not scheduled.");
    }
    

    /**
       Planifica el proceso en el tiempo absoluto de simulacin que recibe como 
       primer parmetro.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.

       @param AtTime Es un nmero 'double' que debe ser mayor o igual que el 
       tiempo actual de la simulacin.
       @param prior Es un 'booleano'. Si es 'true' y hay varios procesos 
       planificados para este tiempo absoluto de simulacin, este proceso 
       ser insertado el primero en la cola en el tiempo absoluto de 
       simulacin 'AtTime'. Si es 'false' y hay varios procesos planificados 
       para este tiempo absoluto de simulacin, este proceso ser insertado el 
       ltimo para el tiempo 'AtTime'.
       
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro AtTime no es mayor o 
       igual que el tiempo actual de simulacin.
       @exception RestartException Si se est reiniciando.
    */
    
    public void ActivateAt (double AtTime, boolean prior) throws SimulationException, RestartException {
	if (Terminated || !idle()) return;
	
	if (AtTime < SimulationProcess.CurrentTime())
	    throw new SimulationException("Invalid time "+AtTime);
	
	Passivated = false;
	wakeuptime = AtTime;
	Scheduler.ReadyQueue.Insert(this, prior);
    }

    /**
       Planifica el proceso en el tiempo absoluto de simulacin que recibe como 
       parmetro.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.

       @param AtTime Es un nmero 'double' que debe ser mayor o igual que el 
       tiempo actual de la simulacin.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro AtTime no es mayor o 
       igual que el tiempo actual de simulacin.
       @exception RestartException Si se est reiniciando.
    */
    
    public void ActivateAt (double AtTime) throws SimulationException, RestartException {
	ActivateAt(AtTime, false);
    }


    /**
       Planifica el proceso en el tiempo de simulacin absoluto que se obtiene 
       sumando el tiempo absoluto de simulacin actual y el tiempo de retardo
       que recibe como primer parmetro.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.

       @param Delay Es un nmero 'double' que debe ser mayor o igual que cero.
       @param prior Es un 'booleano'. Si es 'true' y hay varios procesos 
       planificados para este tiempo absoluto de simulacin, este proceso 
       ser insertado el primero en la cola para el tiempo de simulacin 
       absoluto suma del tiempo absoluto actual y el tiempo de retraso. Si 
       es 'false' y hay varios procesos planificados para este tiempo de 
       simulacin absoluto, este proceso ser insertado el ltimo para ese 
       tiempo absoluto de simulacin.

       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro 'Delay' no es mayor o 
       igual que cero.
       @exception RestartException Si se est reiniciando.
    */
    
    public void ActivateDelay (double Delay, boolean prior) throws SimulationException, RestartException {
	if (Terminated || !idle()) return;
	
	if (!checkTime(Delay))
	    throw new SimulationException("Invalid delay time "+Delay);
	
	Passivated = false;
	wakeuptime = Scheduler.SimulatedTime + Delay;
	Scheduler.ReadyQueue.Insert(this, prior);
    }

    /**
       Planifica el proceso en el tiempo de simulacin absoluto que se obtiene 
       sumando el tiempo absoluto de simulacin actual y el tiempo de retardo
       que recibe como parmetro.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.

       @param Delay Es un nmero 'double' que debe ser mayor o igual que cero.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro 'Delay' no es mayor o 
       igual que cero.
       @exception RestartException Si se est reiniciando.

    */
    
    public void ActivateDelay (double Delay) throws SimulationException, RestartException {
	ActivateDelay(Delay, false);
    }



    /**
       Planifica el proceso en el tiempo de simulacin absoluto actual.
       Este mtodo solo tiene efecto si se ejecuta sobre un objeto proceso que 
       est en estado pasivo.
       El objeto proceso pasa del estado pasivo al estado planificado.

       @return No devuelve ningn valor.
       @exception SimulationException  Nunca se lanza?, pues idle() no la 
       genera, e Insert() y CurrentTime() tampoco.
       @exception RestartException Si se est reiniciando.
    */
    
    public void Activate () throws SimulationException, RestartException {       
	if (Terminated || !idle()) return;
	
        Passivated = false;
        wakeuptime = CurrentTime();
	Scheduler.ReadyQueue.Insert(this, true);
    }
    

     /**
       Replanifica el proceso antes que el objeto proceso que recibe como 
       parmetro y en el mismo tiempo absoluto de simulacin que tiene ste ltimo.
       Este mtodo se puede ejecutar sobre un objeto proceso que 
       est en estado pasivo, activo o planificado.

       Si el objeto est activo podr ejecutar este mtodo sobre s mismo, con lo cual el objeto se replanificara ms tarde.

       El objeto proceso que recibe como parmetro solo puede estar en el 
       estado planificado.
       
       @param p Objeto de una subclase de 'SimulationProcess'.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro es un objeto proceso 
       que no est en el estado planificado.
       @exception RestartException Si se est reiniciando.       
     */
    
    public void ReActivateBefore (SimulationProcess p) throws SimulationException, RestartException {
	if (!idle())
	    Scheduler.unschedule(this);

	ActivateBefore(p);
	if (SimulationProcess.Current == this)
	    Suspend();
    }


    /**
       Reactiva el proceso despus del objeto proceso que recibe como 
       parmetro, en el mismo tiempo absoluto de simulacin de ste ltimo.
       Este mtodo se ejecuta sobre un objeto proceso que 
       est en estado pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo, 
       lo cual har que sea replanificado.
              
       El objeto proceso que recibe como parmetro solo puede estar en el 
       estado planificado.
       
       @param p Objeto de una subclase de 'SimulationProcess'.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro es un objeto proceso 
       que no est en el estado planificado.
       @exception RestartException Si se est reiniciando.    
    */
    
    public void ReActivateAfter  (SimulationProcess p) throws SimulationException, RestartException {
	if (!idle())
	    Scheduler.unschedule(this);
	
	ActivateAfter(p);
	if (SimulationProcess.Current == this)
	    Suspend();
    }


    /**
       Reactiva el proceso para el tiempo absoluto de simulacin que 
       se le pasa como primer parmetro.
       
       Este mtodo se ejecuta sobre un objeto proceso que est en estado 
       pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo,
       lo cual provocar su replanificacin. 
       
       @param AtTime Es un nmero 'double' que debe ser mayor o igual que el 
       tiempo actual de la simulacin.
       @param prior Es un 'booleano'. Si es 'true' y hay varios procesos 
       planificados para este tiempo absoluto de simulacin, este proceso ser 
       insertado el primero en la cola en el tiempo absoluto de simulacin 
       'AtTime'.
       Si es 'false' y hay varios procesos planificados para este
       tiempo absoluto de simulacin, este proceso ser insertado el 
       ltimo para el tiempo 'AtTime'.
       @return No devuelve ningn valor
       @exception SimulationException Si el parmetro 'AtTime' no es mayor o 
       igual que el tiempo actual de simulacin.
       @exception RestartException Si se est reiniciando.
    */
    
    public void ReActivateAt (double AtTime, boolean prior) throws SimulationException, RestartException {
	if (!idle())
	    Scheduler.unschedule(this);
	
	ActivateAt(AtTime, prior);
	
	if (SimulationProcess.Current == this) {
	    Suspend();
	}
    }

    
    /**
       Reactiva el proceso para el tiempo absoluto de simulacin que 
       se le pasa como primer parmetro.
       
       Este mtodo se ejecuta sobre un objeto proceso que est en estado 
       pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo,
       lo cual provocar su replanificacin. 
       
       @param AtTime Es un nmero 'double' que debe ser mayor o igual que el 
       tiempo actual de la simulacin.
       @return No devuelve ningn valor
       @exception SimulationException Si el parmetro 'AtTime' no es mayor o 
       igual que el tiempo actual de simulacin.
       @exception RestartException Si se est reiniciando.

    */

    public void ReActivateAt (double AtTime) throws SimulationException, RestartException {
	ReActivateAt(AtTime, false);
    }


    /**
       Reactiva el proceso para el tiempo de simulacin absoluto que se 
       obtiene sumando el tiempo absoluto de simulacin actual y el tiempo de 
       retardo que recibe como primer parmetro.

       Este mtodo se ejecuta sobre un objeto proceso que est en estado 
       pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo, 
       lo cual har que sea replanificado.

       @param Delay Es un nmero 'double' que debe ser mayor o igual que cero.
       @param prior Es un 'booleano'. Si es 'true' y hay varios procesos 
       planificados para este tiempo de simulacin absoluto, este proceso 
       ser insertado el primero en la cola para el 
       tiempo absoluto de simulacin actual ms el tiempo de retardo.       
       Si es 'false' y hay varios procesos planificados para este tiempo de 
       simulacin absoluto, este proceso ser insertado el ltimo para ese 
       tiempo absoluto de simulacin.

       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro Delay no es mayor o 
       igual que cero.
       @exception RestartException Si se est reiniciando.
    */

    public void ReActivateDelay (double Delay, boolean prior) throws SimulationException, RestartException {
	if (!idle())
	    Scheduler.unschedule(this);

	ActivateDelay(Delay, prior);
	if (SimulationProcess.Current == this)
	    Suspend();
    }

    
    /**
       Reactiva el proceso para el tiempo de simulacin absoluto que se 
       obtiene sumando el tiempo absoluto de simulacin actual y el tiempo de 
       retardo que recibe como primer parmetro.
       
       Este mtodo se ejecuta sobre un objeto proceso que est en estado 
       pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo, 
       lo cual har que sea replanificado.
       
       @param Delay Es un nmero 'double' que debe ser mayor o igual que cero.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro Delay no es mayor o 
       igual que cero.
       @exception RestartException Si se est reiniciando.

    */

    public void ReActivateDelay (double Delay) throws SimulationException, RestartException {
	ReActivateDelay(Delay, false);
    }



    /**
       Replanifica el proceso en el tiempo absoluto de simulacin actual.
       
       Este mtodo se ejecuta sobre un objeto proceso que est en estado 
       pasivo, activo o planificado.
       
       Si el objeto est activo podr ejecutar este mtodo sobre s mismo, 
       el cual har que se replanifique para el tiempo absoluto actual.
       
       @return No devuelve ningn valor
       @exception SimulationException Se produce internamente por la llamada al
       mtodo Activate.
       @exception RestartException Si se est reiniciando
    */

    public void ReActivate () throws SimulationException, RestartException {
	if (!idle())
	    Scheduler.unschedule(this);

	Activate();
	if (SimulationProcess.Current == this)
	    Suspend();
    }
    

    /**
       Cancela las prximas acciones que realizara el proceso. 
       Pasa el objeto proceso del estado activo o del estado planificado al 
       estado pasivo. Para que vuelva a ejecutarse algn otro proceso deber 
       planificarlo mediante algn mtodo del tipo activate o reactivate. 
       
       @return No devuelve ningn valor.
       @exception RestartException Si se est reiniciando.
    */

    public void Cancel () throws RestartException    {
	/*
	 * We must suspend this process either by removing it from
	 * the scheduler queue (if it is already suspended) or by
	 * calling suspend directly.
	 */
	
	if (!idle()) {  // process is running or on queue to be run
	    // currently active, so simply suspend
	    
	    if (this == SimulationProcess.Current) {
		wakeuptime = SimulationProcess.Never;
		Passivated = true;
		Suspend();
	    }
	    else {
		Scheduler.unschedule(this); // remove from queue
	    }
	}
    }
    

    /** 
	Pasa el objeto proceso al estado terminado, el objeto 
	proceso que lo ejecuta no podr	volver al estado activo, ni al
	planificado, ni al pasivo.
	
	@return No devuelve ningn valor.
    */
    
    public void terminate () {
	if (!Terminated) {
	    Terminated = Passivated = true;
	    wakeuptime = SimulationProcess.Never;
	    
	    if ((this != SimulationProcess.Current) && (!idle()))
		Scheduler.unschedule(this);
	    
	    try {
		Scheduler.schedule();
	    }
	    catch (SimulationException e)  { }	   

	    SimulationProcess.allProcesses.Remove(this);
	    //kk  modificado stop();
	}
    }


    /**
       Devuelve un 'booleano'. Si el proceso est en el estado 
       activo o planificado retorna 'false' y 'true' en otro caso.
       
       @return boolean
    */

    public synchronized boolean idle () {
	if (wakeuptime >= SimulationProcess.CurrentTime())
	    return false;
	else
	    return true;
    }
    


    /**
       Devuelve un 'booleano'. Si el proceso est en el 
       estado pasivo o en el estado terminado, retorna 'true'; sino 'false'.
       
       @return boolean       
    */

    public boolean passivated () {
	return Passivated;
    }
    
    
    /**
       Devuelve un 'booleano'. Si el proceso est en el
       estado terminado retorna 'true', en otro caso retorna 'false'.
       @return boolean 
    */
    
    public boolean terminated ()    {
	return Terminated;
    }    

    /**
       Devuelve el proceso actual que est en el estado activo.
       
       @return SimulationProccess Devuelve un objeto proceso de una subclase 
       de SimulationProcess.
       @exception SimulationException Indica que no hay ningn objeto proceso
       en el estado activo.
    */
    
    public static SimulationProcess current () throws SimulationException {
	if (SimulationProcess.Current == null)
	    throw new SimulationException("Current not set.");
	
	return SimulationProcess.Current;
    }
    

    /** 
	Devuelve el tiempo absoluto actual de simulacin que marca el reloj 
	de tiempo virtual de la simulacin.
	
	@return double
    */

    public static double CurrentTime () {
	return Scheduler.SimulatedTime;
    }

    /**
       Crea un objeto de la clase SimulationProccess
       y lo deja en el estado pasivo.
       Al ser un constructor protected se deben crear subclases
       para poder usar la clase SimulationProcess.
    */

    protected SimulationProcess ()    {
	wakeuptime = SimulationProcess.Never;
	Terminated = false;
	Passivated = true;
	started = false;
	
	SimulationProcess.allProcesses.Insert(this);
    }


    /**
       Actualiza el tiempo absoluto de simulacin de un objeto proceso.
       El objeto proceso debe estar en el estado planificado o pasivo.
       @param time Es un 'double' con un tiempo absoluto de simulacin mayor 
       o igual que el actual.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro 'time' es menor 
       que el tiempo de simulacin absoluto actual o si el proceso est 
       en el estado activo.
    */
    
    protected void set_evtime (double time) throws SimulationException    {
	if (!idle()){
	    if (time >= SimulationProcess.CurrentTime())
		wakeuptime = time;
	    else
		throw new SimulationException("Time "+time+" invalid.");
	}
	else
	    throw new SimulationException("SimulationProcess is not idle.");
    }
    
    
    /**
       Detiene el proceso actual las unidades de tiempo relativo de simulacin 
       especificadas en el parmetro.
       Este mtodo es la combinacin de las funciones ActivateDelay(t,false) y 
       Suspend().
       El proceso que ejecuta este mtodo solo puede estar en el estado activo.
       
       @param t Es un 'double', con el tiempo de retardo de simulacin que se 
       detendr el proceso.
       @return No devuelve ningn valor.
       @exception SimulationException Si el parmetro 't' no es mayor o igual 
       que cero.
       
      
       @exception RestartException Si se est reiniciando.
    */
    
    /* (excepcion inactive object), no se produce la excepcin pq es protected
       y dar error de compilacin si lo usa un proceso que no es activo. 
       No se puede hacer HOLD de un objeto que no sea  l mismo.
    */
    protected void Hold (double t) throws SimulationException, RestartException {
	if ((this == SimulationProcess.Current) || (SimulationProcess.Current == null)) {
	    wakeuptime = SimulationProcess.Never;
	    ActivateDelay(t, false);
	    Suspend();
	}
	else
	    throw new SimulationException("Hold applied to inactive object.");
    }
    
    /**
       Pasa el proceso actual del estado activo al estado pasivo.
       Solo saldr de este estado si lo planifica o lo activa
       otro proceso.
       @return No devuelve ningn valor.
       @exception RestartException Si se est reiniciando.
    */
    protected void Passivate () throws RestartException {
	
	if (!Passivated && (this == SimulationProcess.Current))
	    Cancel();
    }

    /**
       Suspende el proceso actual. Solo se puede llamar a este mtodo si el 
       proceso est en el estado activo.
       Es usada por otras funciones de esta clase, el usuario no debe 
       utilizarla directamente.
       
       @return No devuelve ningn valor.
       @exception RestartException Si se est reiniciando.
    */

    protected void Suspend () throws RestartException {
	
	try {
	    if (Scheduler.schedule()) {
		// si cambio de proceso de p1 a p2 hago que p1 haga un wait;
		// en otro caso, al devolver false no hace nada, seguira p1.
		synchronized (mutex) {	        
		    count--;
		    if (count == 0) {
			try {
			    mutex.wait();
			}
			catch (Exception e) {}
		    }

		}
	    }
	}
	catch (SimulationException e) {}
	
	if (Scheduler.simulationReset())
	    throw new RestartException();
    }

    /**
       Despierta el proceso previamente suspendido con Suspend().
       Este mtodo solo puede ser llamado sobre procesos que estn en el 
       estado pasivo, es decir, que acaban de ser creados o que han sido 
       suspendidos previamente.

       Se debe usar nicamente por la clase Scheduler, el usuario no debe 
       utilizarla directamente.

       @return No devuelve ningn valor.
    */

    protected void Resume () {
	/*
	 * To compensate for the initial call to Resume by the
	 * application.
	 */
	
	if (SimulationProcess.Current == null) {
		SimulationProcess.Current = this;
		wakeuptime = SimulationProcess.CurrentTime();
	    }
	
	if (!Terminated){
	    if (!started){
		started = true;
		start();
	    }
	    else {
		synchronized (mutex){
		    count++;

		    if (count >= 0)
			mutex.notify();
		}
	    }
	}
    }

    private boolean checkTime (double time) {
	
	if (time >= 0)
	    return true;
	else
	    return false;
    }
    /** 
	Actualiza las variables del proceso para que stas indiquen que
	el proceso est en el estado pasivo. Se debe usar nicamente por 
	la clase Scheduler, el usuario no debe usarla directamente.

	@return No devuelve ningn valor.
    */
    protected void passivate () {
	
	Passivated = true;
	wakeuptime = SimulationProcess.Never;
    }

    /** 
	Constante que se asigna a la variable wakeuptime cuando el proceso 
	est en el estado pasivo. No tiene sentido su uso por parte del 
	usuario.
    */
    public static final int Never = -1;

    /** 
	Variable de clase que contiene la lista de objetos proceso creados. 
	No tiene sentido su uso por parte del usuario.
    */

    protected static SimulationProcessList allProcesses = 
	new SimulationProcessList();

    private double wakeuptime;
    private boolean Terminated;
    private boolean Passivated;
    private boolean started;
    private Object mutex = new Object();
    private int count = 1;
    
    /** 
	Variable que debe usar el usuario para sincronizar los procesos de
	la simulacin con el main.
    */
    public Object cerrojo = new Object();

    /**
       Variable de clase que indica cul es el proceso actual que est en el 
       estado activo.
    */
    static SimulationProcess Current = null;

};
