package arjuna.JavaSim.Statistics;

import java.io.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;


/**
   Esta clase es la base de la clase  histogram, la cual guarda 
   la cuenta exacta de todos los valores introducidos. Un cubo 
   (?bucket) es creado para cada nuevo valor.
   Esto puede ocupar mucho espacio en una simulacin, por lo tanto,
   se proporcionan clases para histogramas con menos precisin.
 */

public class PrecisionHistogram extends Variance {

    /**
       Crea un histograma vaco.
    */
    
public PrecisionHistogram ()    {
	length = 0;
	Head = null;

	reset();
}
    /** 
	Destructor del objeto.
    */
public void finalize ()     {
        reset();
}

    /**
       Aade el parmetro como un valor del histograma. Si el cubo (?bucket) 
       existe, es incrementado, sino se crear un cubo nuevo.
    */
    
public void setValue (double value) throws IllegalArgumentException    {
	super.setValue(value);

	Bucket trail = null;
    
	for (Bucket ptr = Head; ptr != null; trail = ptr, ptr = ptr.cdr())
	{
	    if (ptr.Name() == value)
	    {
		ptr.incrementSize(1);
		return;
	    }
	    else if (ptr.greaterThan(value))
		break;
	}
    
	// we need to add a new bucket.

	add(trail, value, false);
}

    /**
       Vaca el histograma.
    */
    
public void reset ()     {
	if (length > 0)  // delete old list
	{
	    Bucket trail = Head;
	    for (long i = 0; i < length; i++)
	    {
		Head = Head.cdr();
		trail = Head;
	    }
	
	    length = 0;
	    Head = null;
	}
    
	super.reset();
}

    /**
       Devuelve el nmero de cubos (?buckets) del histograma.
    */
    
public long numberOfBuckets ()    {
	return length;
}
    
    /**
       Obtiene el nmero de entradas de un cubo (?bucket), atravs del ndice
       del bucket, que se le pasa como parmetro.
    */
    
    public double sizeByIndex (long index) throws StatisticsException, IllegalArgumentException    {
	Bucket ptr = Head;

	if ((index < 0) || (index > length))
	    throw(new IllegalArgumentException("index out of range."));

	for (long i = 0; (i < index) && (ptr != null); i++)
	    ptr = ptr.cdr();

	if (ptr != null)
	    return ptr.size();

	// we should never get here!

	throw(new StatisticsException("sizeByIndex went off end of list."));
}
    /**
       Obtiene el nmero de entradas de un cubo (?bucket), atravs del nombre 
       del bucket, que se le pasa como parmtro.
    */
public double sizeByName (double name) throws IllegalArgumentException    {
	for (Bucket ptr = Head; ptr != null; ptr = ptr.cdr())
	{
	    if (ptr.Name() == name)
		return ptr.size();

	    if (ptr.greaterThan(name))  // bucket is not present
		break;
	}

	throw(new IllegalArgumentException("Bucket name "+name+" not found."));
}

    /**
       Guarda el estado del histograma en el fichero que recibe como parmetro.
      */
    
public boolean saveState (String fileName) throws IOException    {
	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream oFile = new DataOutputStream(f);

	boolean res = saveState(oFile);

	f.close();

	return res;
}

    /**
       Guarda el estado del histograma en el 'stream' que recibe como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException    {
	oFile.writeLong(length);

	for (Bucket ptr = Head; ptr != null; ptr = ptr.cdr())
	{
	    oFile.writeDouble(ptr.Name());
	    oFile.writeLong(ptr.size());
	}
    
	return super.saveState(oFile);	
}

    /**
       Restaura el estado del histograma del fichero que recibe como parmetro.
    */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream iFile = new DataInputStream(f);

	boolean res = restoreState(iFile);

	f.close();

	return res;	
}

    /**
       Restaura el estado del histograma del 'stream' que recibe como parmetro.       
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException    {
	long numberEntries;
	double bucketName;
	Bucket ptr = null;
    
	reset();

	length = iFile.readLong();
	
	for (int i = 0; i < length; i++)
	{
	    bucketName = iFile.readDouble();
	    numberEntries = iFile.readLong();
	    
	    Bucket toAdd = new Bucket(bucketName, numberEntries);
	    if (Head != null)
		ptr.setCdr(toAdd);
	    else
		Head = toAdd;

	    ptr = toAdd;
	}

	return super.restoreState(iFile);	
}
    
    /**
       Imprime los datos del histograma.
    */
    
public void print ()    {
	if (length == 0)
	    System.out.println("Empty histogram.");
	else
	    for (Bucket ptr = Head; ptr != null; ptr = ptr.cdr())
		System.out.println("Bucket : < "+ptr.Name()+", "+ptr.size()+" >");

	super.print();
}
    
protected boolean isPresent (double value)    {
	try
	{
	    double dummy = sizeByName(value);

	    return true;
	}
	catch (IllegalArgumentException e)
	{
	    return false;
	}
}

protected void create (double value)    {
	Bucket trail = null;
    
	for (Bucket ptr = Head; ptr != null; trail = ptr, ptr = ptr.cdr())
	{
	    if (ptr.Name() == value)
		return;
	    else
	    if (ptr.greaterThan(value))  // bucket is not present
		break;
	}
    
	add(trail, value, true);
}
    
private void add (Bucket addPosition, double value, boolean createOnly)    {
	long initEntries;

	if (createOnly)
	    initEntries = 0;
	else
	    initEntries = 1;
	
	Bucket newBucket = new Bucket(value, initEntries);

	if (addPosition == null)  // head of list
	{
	    newBucket.setCdr(Head);
	    Head = newBucket;
	}
	else
	{
	    newBucket.setCdr(addPosition.cdr());
	    addPosition.setCdr(newBucket);
	}

	length++;
}
    
protected long length;
protected Bucket Head;

};
